"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBuildConfig = void 0;
var tslib_1 = require("tslib");
var rollup_plugin_closure_compiler_1 = tslib_1.__importDefault(require("@ampproject/rollup-plugin-closure-compiler"));
var plugin_commonjs_1 = tslib_1.__importDefault(require("@rollup/plugin-commonjs"));
var plugin_node_resolve_1 = tslib_1.__importDefault(require("@rollup/plugin-node-resolve"));
var plugin_url_1 = tslib_1.__importDefault(require("@rollup/plugin-url"));
var plugin_image_1 = tslib_1.__importDefault(require("@rollup/plugin-image"));
var plugin_json_1 = tslib_1.__importDefault(require("@rollup/plugin-json"));
var plugin_beep_1 = tslib_1.__importDefault(require("@rollup/plugin-beep"));
var plugin_babel_1 = tslib_1.__importDefault(require("@rollup/plugin-babel"));
var rollup_plugin_typescript2_1 = tslib_1.__importDefault(require("rollup-plugin-typescript2"));
var rollup_plugin_postcss_1 = tslib_1.__importDefault(require("rollup-plugin-postcss"));
var rollup_plugin_strip_code_1 = tslib_1.__importDefault(require("rollup-plugin-strip-code"));
var rollup_plugin_filesize_1 = tslib_1.__importDefault(require("rollup-plugin-filesize"));
var rollup_plugin_progress_1 = tslib_1.__importDefault(require("rollup-plugin-progress"));
var rollup_plugin_eslint_1 = require("rollup-plugin-eslint");
var autoprefixer_1 = tslib_1.__importDefault(require("autoprefixer"));
var cssnano_1 = tslib_1.__importDefault(require("cssnano"));
var postcss_simple_vars_1 = tslib_1.__importDefault(require("postcss-simple-vars"));
var postcss_nested_1 = tslib_1.__importDefault(require("postcss-nested"));
var utils_1 = require("../../../shared/utils");
exports.createBuildConfig = function (_a) {
    var packageJson = _a.packageJson, runEslint = _a.runEslint, displayFilesize = _a.displayFilesize, useClosure = _a.useClosure;
    return ({
        input: packageJson.source,
        output: {
            file: packageJson.module,
            format: 'es',
            sourcemap: true
        },
        external: utils_1.getExternalScreen(packageJson),
        plugins: [
            plugin_beep_1.default(),
            rollup_plugin_progress_1.default({ clearLine: true }),
            runEslint && rollup_plugin_eslint_1.eslint(),
            displayFilesize && rollup_plugin_filesize_1.default(),
            useClosure && rollup_plugin_closure_compiler_1.default(),
            plugin_json_1.default(),
            plugin_url_1.default(),
            plugin_image_1.default(),
            rollup_plugin_postcss_1.default({
                extract: false,
                modules: true,
                writeDefinitions: true,
                plugins: [
                    postcss_simple_vars_1.default({ variables: {} }),
                    postcss_nested_1.default(),
                    cssnano_1.default(),
                    autoprefixer_1.default()
                ],
                extensions: ['.css', '.scss', '.sass'],
                use: ['sass']
            }),
            rollup_plugin_typescript2_1.default({
                clean: true,
                tsconfigDefaults: {
                    exclude: [
                        '**/*.spec.ts',
                        '**/*.test.ts',
                        '**/*.spec.tsx',
                        '**/*.test.tsx',
                        'node_modules',
                        'dist'
                    ],
                    compilerOptions: {
                        rootDir: './src',
                        sourceMap: true,
                        declaration: true,
                        jsx: 'react',
                        target: 'es5',
                        lib: ['dom', 'dom.iterable', 'esnext'],
                        allowJs: true,
                        skipLibCheck: true,
                        esModuleInterop: true,
                        allowSyntheticDefaultImports: true,
                        strict: true,
                        forceConsistentCasingInFileNames: true,
                        module: 'esnext',
                        moduleResolution: 'node',
                        resolveJsonModule: true,
                        isolatedModules: true,
                        noEmit: true
                    }
                }
            }),
            plugin_commonjs_1.default({ include: /\/node_modules\// }),
            plugin_node_resolve_1.default({
                extensions: ['.js', '.jsx', '.ts', '.tsx'],
                preferBuiltins: false,
                browser: true
            }),
            plugin_babel_1.default({
                babelHelpers: 'runtime',
                skipPreflightCheck: true,
                babelrc: false,
                extensions: ['.js', 'jsx', '.ts', '.tsx'],
                presets: ['@babel/preset-env', '@babel/preset-react'],
                plugins: [
                    '@babel/plugin-transform-runtime',
                    '@babel/proposal-class-properties'
                ],
                exclude: /\/node_modules\//
            }),
            rollup_plugin_strip_code_1.default({
                start_comment: 'placeholder:start',
                end_comment: 'placeholder:end'
            })
        ]
    });
};
