"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.installBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
exports.installBinCommand = function (prog) {
    prog
        .command('install')
        .describe('Install one or more dependencies to the workspace root. Run this script inside any package and re-space will add peer dependencies as well')
        .alias('i')
        .example('install libraryName')
        .option('dev, D', 'Install development dependencies')
        .example("install libraryName --dev")
        .action(function (_a) {
        var dependenciesList = _a._, dev = _a.dev;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var dependencies, installSpinner, workspacePackage, installPackageArgs, dependenciesList_1, dependenciesList_1_1, dependency, _b, workspaceRoot, installRootArgs, dependenciesList_2, dependenciesList_2_1, dependency, err_1;
            var e_1, _c, e_2, _d;
            return tslib_1.__generator(this, function (_e) {
                switch (_e.label) {
                    case 0:
                        dependencies = dependenciesList.join(' ');
                        installSpinner = ora_1.default(messages_1.installMessage.installing(dependencies));
                        installSpinner.start();
                        _e.label = 1;
                    case 1:
                        _e.trys.push([1, 4, , 5]);
                        return [4 /*yield*/, utils_1.findWorkspacePackageDir(true)];
                    case 2:
                        workspacePackage = _e.sent();
                        installPackageArgs = ['add'];
                        if (dev) {
                            installPackageArgs.push('--dev');
                        }
                        else {
                            installPackageArgs.push('--peer');
                        }
                        try {
                            for (dependenciesList_1 = tslib_1.__values(dependenciesList), dependenciesList_1_1 = dependenciesList_1.next(); !dependenciesList_1_1.done; dependenciesList_1_1 = dependenciesList_1.next()) {
                                dependency = dependenciesList_1_1.value;
                                installPackageArgs.push(dependency);
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (dependenciesList_1_1 && !dependenciesList_1_1.done && (_c = dependenciesList_1.return)) _c.call(dependenciesList_1);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                        return [4 /*yield*/, execa_1.default('yarn', installPackageArgs, { cwd: workspacePackage })];
                    case 3:
                        _e.sent();
                        return [3 /*break*/, 5];
                    case 4:
                        _b = _e.sent();
                        return [3 /*break*/, 5];
                    case 5:
                        _e.trys.push([5, 8, , 9]);
                        return [4 /*yield*/, utils_1.findWorkspaceRootDir(true)];
                    case 6:
                        workspaceRoot = _e.sent();
                        installRootArgs = ['add', '--exact', '-W'];
                        if (dev) {
                            installRootArgs.push('--dev');
                        }
                        if (typeof dev === 'string') {
                            dependenciesList.push(dev);
                        }
                        try {
                            for (dependenciesList_2 = tslib_1.__values(dependenciesList), dependenciesList_2_1 = dependenciesList_2.next(); !dependenciesList_2_1.done; dependenciesList_2_1 = dependenciesList_2.next()) {
                                dependency = dependenciesList_2_1.value;
                                installRootArgs.push(dependency);
                            }
                        }
                        catch (e_2_1) { e_2 = { error: e_2_1 }; }
                        finally {
                            try {
                                if (dependenciesList_2_1 && !dependenciesList_2_1.done && (_d = dependenciesList_2.return)) _d.call(dependenciesList_2);
                            }
                            finally { if (e_2) throw e_2.error; }
                        }
                        return [4 /*yield*/, execa_1.default('yarn', installRootArgs, { cwd: workspaceRoot })];
                    case 7:
                        _e.sent();
                        installSpinner.succeed(messages_1.installMessage.successful(dependencies));
                        return [3 /*break*/, 9];
                    case 8:
                        err_1 = _e.sent();
                        installSpinner.fail(messages_1.installMessage.failed(dependencies));
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 9];
                    case 9: return [2 /*return*/];
                }
            });
        });
    });
};
