"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lintBinCommand = void 0;
var tslib_1 = require("tslib");
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var path_1 = tslib_1.__importDefault(require("path"));
var eslint_1 = require("eslint");
var lint_config_1 = require("./configs/lint.config");
var messages_1 = require("../../shared/messages");
var package_const_1 = require("../../shared/constants/package.const");
var lint_helpers_1 = require("./lint.helpers");
exports.lintBinCommand = function (prog) {
    prog
        .command('lint')
        .describe('Lint a package (default lint pattern src/**/*.{js,jsx,ts,tsx}')
        .alias('l')
        .option('fix', 'Resolve fixable eslint errors')
        .example('lint --fix')
        .option('ignore-pattern', 'Ignore a pattern')
        .example('lint --ignore-pattern src/foo.ts')
        .action(function (opts) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var time, files, _a, dir, project, packageJsonPath, _b, eslintConfig, cli, report, duration;
        return tslib_1.__generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    time = process.hrtime();
                    files = opts._.length > 0 ? opts._ : ['src/**/*.{js,jsx,ts,tsx}'];
                    return [4 /*yield*/, lint_helpers_1.getPackageLintInfo()];
                case 1:
                    _a = _c.sent(), dir = _a.dir, project = _a.project;
                    packageJsonPath = path_1.default.resolve(dir, package_const_1.PACKAGE_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 2:
                    _b = (_c.sent()).eslintConfig, eslintConfig = _b === void 0 ? {} : _b;
                    cli = new eslint_1.CLIEngine({
                        baseConfig: tslib_1.__assign(tslib_1.__assign({}, lint_config_1.createLintConfig(dir)), eslintConfig),
                        extensions: ['.ts', '.tsx', '.js', '.jsx'],
                        fix: opts.fix,
                        ignorePattern: opts['ignore-pattern'],
                        parser: '@typescript-eslint/parser',
                        parserOptions: {
                            tsconfigRootDir: dir,
                            project: project
                        }
                    });
                    console.log(messages_1.lintMessage.linting(files));
                    report = cli.executeOnFiles(files);
                    if (opts.fix) {
                        eslint_1.CLIEngine.outputFixes(report);
                    }
                    console.log(cli.getFormatter()(report.results));
                    duration = process.hrtime(time);
                    console.log(messages_1.lintMessage.linted(duration));
                    if (report.errorCount) {
                        process.exit(1);
                    }
                    return [2 /*return*/];
            }
        });
    }); });
};
