"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var add_1 = require("./setup/add");
var package_const_1 = require("../../shared/constants/package.const");
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var add_helpers_1 = require("./add.helpers");
var featureOptions = Object.keys(add_1.featureSetup);
exports.addBinCommand = function (prog) {
    prog
        .command('add [featureName]')
        .describe("Add available feature [" + featureOptions.join(', ') + "]")
        .example('add')
        .example('add playground')
        .action(function (featureName) {
        if (featureName === void 0) { featureName = ''; }
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var featureOption, packageDir, packageJsonPath, packageJson, bootSpinner, err_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, add_helpers_1.validateFeatureOption(featureName, featureOptions, function () { return console.log(messages_1.addMessage.invalidFeatureName(featureOption)); })];
                    case 1:
                        featureOption = _a.sent();
                        return [4 /*yield*/, utils_1.findWorkspacePackageDir()];
                    case 2:
                        packageDir = _a.sent();
                        packageJsonPath = path_1.default.resolve(packageDir, package_const_1.PACKAGE_JSON);
                        return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                    case 3:
                        packageJson = (_a.sent());
                        bootSpinner = ora_1.default(messages_1.addMessage.adding(featureOption));
                        bootSpinner.start();
                        _a.label = 4;
                    case 4:
                        _a.trys.push([4, 7, , 8]);
                        return [4 /*yield*/, add_helpers_1.copyFeatureTemplate(packageDir, featureOption)];
                    case 5:
                        _a.sent();
                        return [4 /*yield*/, add_helpers_1.addFeatureScriptsToPackageJson({
                                dir: packageJsonPath,
                                packageJson: packageJson,
                                option: featureOption
                            })];
                    case 6:
                        _a.sent();
                        bootSpinner.succeed(messages_1.addMessage.successful(featureOption));
                        return [3 /*break*/, 8];
                    case 7:
                        err_1 = _a.sent();
                        bootSpinner.fail(messages_1.addMessage.failed(featureOption));
                        if (err_1.toString().includes('already exists')) {
                            console.log(messages_1.addMessage.exists());
                        }
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 8];
                    case 8: return [2 /*return*/];
                }
            });
        });
    });
};
