"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.newBinCommand = void 0;
var tslib_1 = require("tslib");
var ora_1 = tslib_1.__importDefault(require("ora"));
var path_1 = tslib_1.__importDefault(require("path"));
var utils_1 = require("../../shared/utils");
var scaffolding_helpers_1 = require("./scaffolding.helpers");
var new_1 = require("./setup/new");
var messages_1 = require("../../shared/messages");
var new_helpers_1 = require("./new.helpers");
var templateOptions = Object.keys(new_1.newSetup);
exports.newBinCommand = function (prog) {
    prog
        .command('new <name> [dir]')
        .describe("Create a new project.\n    Project name is required.\n    Directory is optional. However, if you specify dir, then template files will be installed there, even if this folder already exists, but without overwriting.")
        .example('new projectName')
        .example('new projectName .')
        .example('new projectName projectDirectory')
        .option('t, template', "Specify a template.\n     Available templates: [" + templateOptions.join(', ') + "]\n     \n     ", 'cra')
        .example("new projectName --template " + templateOptions[0])
        .option('f, force', 'Skip folder checks', false)
        .action(function (name, dir, _a) {
        var template = _a.template, force = _a.force;
        return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var bootSpinner, initialDir, _b, projectDir, projectName, _c, author, templateConfig, packageJsonPreset, err_1, preparingSpinner;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        bootSpinner = ora_1.default();
                        initialDir = path_1.default.resolve(dir || name);
                        if (!force) return [3 /*break*/, 1];
                        _c = { projectDir: initialDir, projectName: name };
                        return [3 /*break*/, 3];
                    case 1: return [4 /*yield*/, new_helpers_1.resolveOptions({ name: name, dir: initialDir })];
                    case 2:
                        _c = _d.sent();
                        _d.label = 3;
                    case 3:
                        _b = _c, projectDir = _b.projectDir, projectName = _b.projectName;
                        bootSpinner.start(messages_1.newMessage.creating(projectDir));
                        _d.label = 4;
                    case 4:
                        _d.trys.push([4, 8, , 9]);
                        return [4 /*yield*/, scaffolding_helpers_1.copyTemplate({ dir: projectDir, bin: 'new', template: template })];
                    case 5:
                        _d.sent();
                        bootSpinner.stop();
                        return [4 /*yield*/, scaffolding_helpers_1.getAuthor()];
                    case 6:
                        author = _d.sent();
                        scaffolding_helpers_1.setNpmAuthorName(author);
                        bootSpinner.start();
                        process.chdir(projectDir);
                        templateConfig = new_1.newSetup[template];
                        packageJsonPreset = tslib_1.__assign(tslib_1.__assign({}, templateConfig.packageJson), { name: projectName, author: author });
                        return [4 /*yield*/, scaffolding_helpers_1.createPackageJson({
                                dir: projectDir,
                                preset: packageJsonPreset
                            })];
                    case 7:
                        _d.sent();
                        bootSpinner.succeed(messages_1.newMessage.created({
                            dir: projectDir,
                            name: projectName
                        }));
                        return [3 /*break*/, 9];
                    case 8:
                        err_1 = _d.sent();
                        bootSpinner.fail(messages_1.newMessage.failed(projectName));
                        utils_1.logError(err_1);
                        process.exit(1);
                        return [3 /*break*/, 9];
                    case 9:
                        preparingSpinner = ora_1.default(messages_1.newMessage.preparing()).start();
                        try {
                            scaffolding_helpers_1.sortPackageJson();
                            utils_1.installDependencies();
                            preparingSpinner.succeed(messages_1.newMessage.prepared());
                        }
                        catch (err) {
                            preparingSpinner.fail(messages_1.newMessage.failedPreparation());
                            utils_1.logError(err);
                        }
                        console.log(messages_1.newMessage.finish(projectDir));
                        return [2 /*return*/];
                }
            });
        });
    });
};
