"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preventFolderCollisions = exports.copyTemplate = exports.getAuthor = exports.createPackageJson = exports.setNpmAuthorName = exports.sortPackageJson = void 0;
var tslib_1 = require("tslib");
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var path_1 = tslib_1.__importDefault(require("path"));
var shelljs_1 = require("shelljs");
var enquirer_1 = require("enquirer");
var package_const_1 = require("../../shared/constants/package.const");
exports.sortPackageJson = function () {
    return shelljs_1.exec('npx sort-package-json', { silent: true });
};
function setNpmAuthorName(author) {
    shelljs_1.exec("npm config set init-author-name \"" + author + "\"", { silent: true });
}
exports.setNpmAuthorName = setNpmAuthorName;
exports.createPackageJson = function (_a) {
    var dir = _a.dir, preset = _a.preset;
    return fs_extra_1.default.outputJSON(path_1.default.resolve(dir, package_const_1.PACKAGE_JSON), preset, { spaces: 2 });
};
function getNpmAuthorName() {
    var author = '';
    author = shelljs_1.exec('npm config get init-author-name', {
        silent: true
    }).stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.exec('git config user.name', { silent: true }).stdout.trim();
    if (author) {
        setNpmAuthorName(author);
        return author;
    }
    author = shelljs_1.exec('npm config get init-author-email', {
        silent: true
    }).stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.exec('git config user.email', { silent: true }).stdout.trim();
    if (author) {
        return author;
    }
    return author;
}
exports.getAuthor = function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var author, licenseInput;
    return tslib_1.__generator(this, function (_a) {
        author = getNpmAuthorName();
        if (author) {
            return [2 /*return*/, author];
        }
        licenseInput = new enquirer_1.Input({
            name: 'author',
            message: 'Who is the package author?'
        });
        return [2 /*return*/, licenseInput.run()];
    });
}); };
exports.copyTemplate = function (_a) {
    var dir = _a.dir, bin = _a.bin, template = _a.template;
    return fs_extra_1.default.copy(path_1.default.resolve(__dirname, "../../../../templates/" + bin + "/" + template), dir, { overwrite: false });
};
exports.preventFolderCollisions = function (_a) {
    var basePath = _a.basePath, name = _a.name, onPromptMessage = _a.onPromptMessage, onPromptInitial = _a.onPromptInitial;
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var isExist, namePrompt, newProjectName;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    isExist = fs_extra_1.default.existsSync(path_1.default.resolve(basePath, name));
                    if (!isExist) {
                        return [2 /*return*/, name];
                    }
                    namePrompt = new enquirer_1.Input({
                        message: onPromptMessage(name),
                        initial: onPromptInitial(name),
                        result: function (v) { return v.trim(); }
                    });
                    return [4 /*yield*/, namePrompt.run()];
                case 1:
                    newProjectName = _b.sent();
                    return [2 /*return*/, exports.preventFolderCollisions({
                            basePath: basePath,
                            name: newProjectName,
                            onPromptMessage: onPromptMessage,
                            onPromptInitial: onPromptInitial
                        })];
            }
        });
    });
};
