"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesBuildBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var workspaces_helpers_1 = require("./workspaces.helpers");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesBuildBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces build')
        .describe('Build each workspace')
        .example('workspaces build')
        .alias('wb')
        .option('q, quiet', 'Do not print logs', false)
        .example('workspaces build --quiet')
        .option('e, exclude', 'Exclude specific workspaces')
        .example('workspaces build --exclude  workspace1,workspace2,workspace3')
        .action(function (_a) {
        var quiet = _a.quiet, exclude = _a.exclude;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _b, chunks, packagesLocationMap, excluded, time, chunks_1, chunks_1_1, chunk, e_1_1, duration, error_1;
            var e_1, _c;
            var _this = this;
            return tslib_1.__generator(this, function (_d) {
                switch (_d.label) {
                    case 0: return [4 /*yield*/, workspaces_helpers_1.exposeWorkspacesInfo()];
                    case 1:
                        _b = _d.sent(), chunks = _b.chunks, packagesLocationMap = _b.packagesLocationMap;
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        utils_1.clearConsole();
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('build'));
                        if (!quiet) {
                            utils_1.space();
                        }
                        _d.label = 2;
                    case 2:
                        _d.trys.push([2, 11, , 12]);
                        time = process.hrtime();
                        _d.label = 3;
                    case 3:
                        _d.trys.push([3, 8, 9, 10]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _d.label = 4;
                    case 4:
                        if (!!chunks_1_1.done) return [3 /*break*/, 7];
                        chunk = chunks_1_1.value;
                        return [4 /*yield*/, Promise.all(workspaces_helpers_1.withExcludedWorkspaces(chunk, excluded).map(function (name) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var stderr;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            if (!quiet) {
                                                console.log(messages_1.workspacesMessage.running(name));
                                            }
                                            return [4 /*yield*/, execa_1.default('re-space', ['build'], {
                                                    cwd: packagesLocationMap[name]
                                                })];
                                        case 1:
                                            stderr = (_a.sent()).stderr;
                                            if (!quiet) {
                                                // TODO: refactor when build/serveWorkspace ready
                                                if (stderr !== '' && stderr !== undefined) {
                                                    throw new Error(stderr);
                                                }
                                                console.log(messages_1.workspacesMessage.finished('build', name));
                                            }
                                            return [2 /*return*/];
                                    }
                                });
                            }); }))];
                    case 5:
                        _d.sent();
                        _d.label = 6;
                    case 6:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 4];
                    case 7: return [3 /*break*/, 10];
                    case 8:
                        e_1_1 = _d.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 10];
                    case 9:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_c = chunks_1.return)) _c.call(chunks_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 10:
                        if (!quiet) {
                            utils_1.space();
                        }
                        duration = process.hrtime(time);
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 12];
                    case 11:
                        error_1 = _d.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_1);
                        return [3 /*break*/, 12];
                    case 12: return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesBuildBinCommand = workspacesBuildBinCommand;
