"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesLintBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var workspaces_helpers_1 = require("./workspaces.helpers");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesLintBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces lint')
        .describe('Lint each workspace')
        .example('workspaces lint')
        .alias('wl')
        .option('fix', 'Resolve fixable eslint errors', false)
        .example('workspaces lint --fix')
        .option('exclude', 'Exclude specific workspaces', '')
        .example('workspaces lint --exclude workspace1,workspace2,workspace3')
        .action(function (_a) {
        var exclude = _a.exclude, fix = _a.fix;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _b, chunks, packagesLocationMap, excluded, args, time, chunks_1, chunks_1_1, chunk, _c, _d, name, cwd, error_1, e_1_1, e_2_1, duration, error_2;
            var e_2, _e, e_1, _f;
            return tslib_1.__generator(this, function (_g) {
                switch (_g.label) {
                    case 0: return [4 /*yield*/, workspaces_helpers_1.exposeWorkspacesInfo()];
                    case 1:
                        _b = _g.sent(), chunks = _b.chunks, packagesLocationMap = _b.packagesLocationMap;
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        args = ['lint', 'src/**/*.{js,jsx,ts,tsx}'];
                        if (fix) {
                            args.push('--fix');
                        }
                        utils_1.clearConsole();
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('lint'));
                        _g.label = 2;
                    case 2:
                        _g.trys.push([2, 20, , 21]);
                        utils_1.space();
                        time = process.hrtime();
                        _g.label = 3;
                    case 3:
                        _g.trys.push([3, 17, 18, 19]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _g.label = 4;
                    case 4:
                        if (!!chunks_1_1.done) return [3 /*break*/, 16];
                        chunk = chunks_1_1.value;
                        _g.label = 5;
                    case 5:
                        _g.trys.push([5, 13, 14, 15]);
                        _c = (e_1 = void 0, tslib_1.__values(workspaces_helpers_1.withExcludedWorkspaces(chunk, excluded))), _d = _c.next();
                        _g.label = 6;
                    case 6:
                        if (!!_d.done) return [3 /*break*/, 12];
                        name = _d.value;
                        utils_1.space();
                        console.log(messages_1.workspacesMessage.running(name));
                        _g.label = 7;
                    case 7:
                        _g.trys.push([7, 9, , 10]);
                        cwd = packagesLocationMap[name];
                        return [4 /*yield*/, execa_1.default('re-space', args, { cwd: cwd, stdio: 'inherit' })];
                    case 8:
                        _g.sent();
                        return [3 /*break*/, 10];
                    case 9:
                        error_1 = _g.sent();
                        utils_1.logError(error_1);
                        return [3 /*break*/, 10];
                    case 10:
                        console.log(messages_1.workspacesMessage.finished('lint', name));
                        _g.label = 11;
                    case 11:
                        _d = _c.next();
                        return [3 /*break*/, 6];
                    case 12: return [3 /*break*/, 15];
                    case 13:
                        e_1_1 = _g.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 15];
                    case 14:
                        try {
                            if (_d && !_d.done && (_f = _c.return)) _f.call(_c);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 15:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 4];
                    case 16: return [3 /*break*/, 19];
                    case 17:
                        e_2_1 = _g.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 19];
                    case 18:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_e = chunks_1.return)) _e.call(chunks_1);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 19:
                        duration = process.hrtime(time);
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 21];
                    case 20:
                        error_2 = _g.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_2);
                        return [3 /*break*/, 21];
                    case 21: return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesLintBinCommand = workspacesLintBinCommand;
