"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.workspacesTestBinCommand = void 0;
var tslib_1 = require("tslib");
var execa_1 = tslib_1.__importDefault(require("execa"));
var messages_1 = require("../../shared/messages");
var utils_1 = require("../../shared/utils");
var dataStructures_utils_1 = require("../../shared/utils/dataStructures.utils");
var workspaces_helpers_1 = require("./workspaces.helpers");
var package_json_1 = tslib_1.__importDefault(require("../../../package.json"));
function workspacesTestBinCommand(prog) {
    var _this = this;
    prog
        .command('workspaces test')
        .describe('Test each workspace')
        .example('workspaces test')
        .alias('wt')
        .option('exclude', 'Exclude specific workspaces', '')
        .example('workspaces test --exclude workspace1,workspace2,workspace3')
        .action(function (_a) {
        var exclude = _a.exclude;
        return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var _b, chunks, packagesLocationMap, excluded, time, chunks_1, chunks_1_1, chunk, _c, _d, name, e_1_1, e_2_1, duration, error_1;
            var e_2, _e, e_1, _f;
            return tslib_1.__generator(this, function (_g) {
                switch (_g.label) {
                    case 0: return [4 /*yield*/, workspaces_helpers_1.exposeWorkspacesInfo()];
                    case 1:
                        _b = _g.sent(), chunks = _b.chunks, packagesLocationMap = _b.packagesLocationMap;
                        excluded = dataStructures_utils_1.convertStringArrayIntoMap(exclude);
                        excluded.set(package_json_1.default.name, true);
                        utils_1.clearConsole();
                        console.log(messages_1.workspacesMessage.introduce());
                        console.log(messages_1.workspacesMessage.started('test'));
                        _g.label = 2;
                    case 2:
                        _g.trys.push([2, 17, , 18]);
                        utils_1.space();
                        time = process.hrtime();
                        _g.label = 3;
                    case 3:
                        _g.trys.push([3, 14, 15, 16]);
                        chunks_1 = tslib_1.__values(chunks), chunks_1_1 = chunks_1.next();
                        _g.label = 4;
                    case 4:
                        if (!!chunks_1_1.done) return [3 /*break*/, 13];
                        chunk = chunks_1_1.value;
                        _g.label = 5;
                    case 5:
                        _g.trys.push([5, 10, 11, 12]);
                        _c = (e_1 = void 0, tslib_1.__values(workspaces_helpers_1.withExcludedWorkspaces(chunk, excluded))), _d = _c.next();
                        _g.label = 6;
                    case 6:
                        if (!!_d.done) return [3 /*break*/, 9];
                        name = _d.value;
                        utils_1.space();
                        console.log(messages_1.workspacesMessage.running(name));
                        return [4 /*yield*/, execa_1.default('re-space', ['test', '--passWithNoTests'], {
                                cwd: packagesLocationMap[name],
                                stdio: 'inherit'
                            })];
                    case 7:
                        _g.sent();
                        console.log(messages_1.workspacesMessage.finished('test', name));
                        _g.label = 8;
                    case 8:
                        _d = _c.next();
                        return [3 /*break*/, 6];
                    case 9: return [3 /*break*/, 12];
                    case 10:
                        e_1_1 = _g.sent();
                        e_1 = { error: e_1_1 };
                        return [3 /*break*/, 12];
                    case 11:
                        try {
                            if (_d && !_d.done && (_f = _c.return)) _f.call(_c);
                        }
                        finally { if (e_1) throw e_1.error; }
                        return [7 /*endfinally*/];
                    case 12:
                        chunks_1_1 = chunks_1.next();
                        return [3 /*break*/, 4];
                    case 13: return [3 /*break*/, 16];
                    case 14:
                        e_2_1 = _g.sent();
                        e_2 = { error: e_2_1 };
                        return [3 /*break*/, 16];
                    case 15:
                        try {
                            if (chunks_1_1 && !chunks_1_1.done && (_e = chunks_1.return)) _e.call(chunks_1);
                        }
                        finally { if (e_2) throw e_2.error; }
                        return [7 /*endfinally*/];
                    case 16:
                        utils_1.space();
                        duration = process.hrtime(time);
                        console.log(messages_1.workspacesMessage.successful(duration));
                        utils_1.space();
                        return [3 /*break*/, 18];
                    case 17:
                        error_1 = _g.sent();
                        console.log(messages_1.workspacesMessage.failed());
                        utils_1.logError(error_1);
                        return [3 /*break*/, 18];
                    case 18: return [2 /*return*/];
                }
            });
        });
    });
}
exports.workspacesTestBinCommand = workspacesTestBinCommand;
