"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExternalScreen = exports.splitWorkspacesIntoDependencyGraph = exports.readWorkspacePackages = void 0;
var tslib_1 = require("tslib");
var fs_utils_1 = require("./fs.utils");
exports.readWorkspacePackages = function (packagesInfo) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var packageJsons$, settledPackageJsons;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                packageJsons$ = packagesInfo.map(function (pkg) {
                    return fs_utils_1.readPackageJson(pkg.location);
                });
                return [4 /*yield*/, Promise.allSettled(packageJsons$)];
            case 1:
                settledPackageJsons = (_a.sent());
                return [2 /*return*/, settledPackageJsons.map(function (settledPackageJson) { return settledPackageJson.value; })];
        }
    });
}); };
exports.splitWorkspacesIntoDependencyGraph = function (workspaces) {
    var e_1, _a;
    var chunks = [];
    var workspacePackagesMap = Object.fromEntries(workspaces.map(function (_a) {
        var name = _a.name, _b = _a.dependencies, dependencies = _b === void 0 ? {} : _b, _c = _a.devDependencies, devDependencies = _c === void 0 ? {} : _c, _d = _a.peerDependencies, peerDependencies = _d === void 0 ? {} : _d;
        return [
            name,
            tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, dependencies), devDependencies), peerDependencies)
        ];
    }));
    var packageDependenciesMap = new Map();
    for (var workspacePackage in workspacePackagesMap) {
        packageDependenciesMap.set(workspacePackage, Object.keys(workspacePackagesMap[workspacePackage]).filter(function (dependency) { return workspacePackagesMap[dependency] !== undefined; }));
    }
    var currentChunk = Array.from(packageDependenciesMap.keys()).reduce(function (acc, name) {
        var hasWorkspaceDependencies = packageDependenciesMap.get(name).length > 0;
        return hasWorkspaceDependencies ? acc : tslib_1.__spread(acc, [name]);
    }, []);
    while (currentChunk.length > 0) {
        chunks.push(currentChunk);
        try {
            for (var currentChunk_1 = (e_1 = void 0, tslib_1.__values(currentChunk)), currentChunk_1_1 = currentChunk_1.next(); !currentChunk_1_1.done; currentChunk_1_1 = currentChunk_1.next()) {
                var name = currentChunk_1_1.value;
                packageDependenciesMap.delete(name);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (currentChunk_1_1 && !currentChunk_1_1.done && (_a = currentChunk_1.return)) _a.call(currentChunk_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        currentChunk = Array.from(packageDependenciesMap.keys()).filter(function (pkg) {
            var _a, _b;
            var deps = (_b = (_a = packageDependenciesMap
                .get(pkg)) === null || _a === void 0 ? void 0 : _a.filter(function (dep) { return packageDependenciesMap.has(dep); })) !== null && _b !== void 0 ? _b : [];
            packageDependenciesMap.set(pkg, deps);
            return deps.length === 0;
        });
    }
    return {
        chunks: chunks,
        unprocessed: Array.from(packageDependenciesMap.entries())
    };
};
exports.getExternalScreen = function (_a) {
    var _b = _a.dependencies, dependencies = _b === void 0 ? {} : _b, _c = _a.peerDependencies, peerDependencies = _c === void 0 ? {} : _c, _d = _a.devDependencies, devDependencies = _d === void 0 ? {} : _d;
    var externals = tslib_1.__spread(Object.keys(dependencies), Object.keys(peerDependencies), Object.keys(devDependencies));
    var externalsMap = new Map(externals.map(function (key) { return [key, key]; }));
    return function (id) {
        return externalsMap.has(id) ||
            Boolean(externals.find(function (key) { return id.startsWith(key + "/"); }));
    };
};
