"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
/* eslint-disable sonarjs/no-duplicate-string */
var dependency_utils_1 = require("./dependency.utils");
describe('[utils.dependency]', function () {
    describe('splitWorkspacesIntoDependencyGraph', function () {
        test('should return chunks when there is no package jsons', function () {
            var packageJsons = [];
            expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                chunks: [],
                unprocessed: []
            });
        });
        test('should return one chunk with one dependency', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {}
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return one chunk with a few straight dependencies', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {}
                    },
                    {
                        name: 'example-2',
                        dependencies: {}
                    },
                    {
                        name: 'example-3',
                        dependencies: {}
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2', 'example-3']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return one chunk with 2 packages without dependencies prop', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2'
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return one chunk with 2 packages with empty dependencies prop', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {}
                    },
                    {
                        name: 'example-2',
                        dependencies: {}
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return one chunk with package without dependencies prop and another package with empty dependencies prop', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2',
                        dependencies: {}
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return two chunks with dependency between 2 packages', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {}
                    },
                    {
                        name: 'example-2',
                        dependencies: {
                            'example-1': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1'], ['example-2']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return two chunks when last needs both two from the first chunk', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2'
                    },
                    {
                        name: 'example-3',
                        dependencies: {
                            'example-1': '*',
                            'example-2': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2'], ['example-3']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should decompose package in chunk separately even when independent of all packages from the previous chunk', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2'
                    },
                    {
                        name: 'example-3',
                        dependencies: {
                            'example-1': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1', 'example-2'], ['example-3']],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return only unprocessed packages when there is nothing to compose due to circular dependency', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {
                            'example-2': '*'
                        }
                    },
                    {
                        name: 'example-2',
                        dependencies: {
                            'example-1': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [],
                    unprocessed: [
                        ['example-1', ['example-2']],
                        ['example-2', ['example-1']]
                    ]
                });
                return [2 /*return*/];
            });
        }); });
        test('should return only unprocessed all packages where there is circular dependency between 3 packages', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1',
                        dependencies: {
                            'example-2': '*'
                        }
                    },
                    {
                        name: 'example-2',
                        dependencies: {
                            'example-3': '*'
                        }
                    },
                    {
                        name: 'example-3',
                        dependencies: {
                            'example-1': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [],
                    unprocessed: [
                        ['example-1', ['example-2']],
                        ['example-2', ['example-3']],
                        ['example-3', ['example-1']]
                    ]
                });
                return [2 /*return*/];
            });
        }); });
        test('should have unprocessed chunk without dependencies which already been processed', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2',
                        dependencies: {
                            'example-1': '*',
                            'example-3': '*'
                        }
                    },
                    {
                        name: 'example-3',
                        dependencies: {
                            'example-1': '*',
                            'example-2': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1']],
                    unprocessed: [
                        ['example-2', ['example-3']],
                        ['example-3', ['example-2']]
                    ]
                });
                return [2 /*return*/];
            });
        }); });
        test('should return processed and unprocessed dependencies when there is a circular dependency in the middle', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'example-1'
                    },
                    {
                        name: 'example-2',
                        dependencies: {
                            'example-1': '*',
                            'example-3': '*'
                        }
                    },
                    {
                        name: 'example-3',
                        dependencies: {
                            'example-2': '*'
                        }
                    },
                    {
                        name: 'example-4',
                        dependencies: {
                            'example-1': '*'
                        }
                    },
                    {
                        name: 'example-5',
                        dependencies: {
                            'example-1': '*',
                            'example-4': '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [['example-1'], ['example-4'], ['example-5']],
                    unprocessed: [
                        ['example-2', ['example-3']],
                        ['example-3', ['example-2']]
                    ]
                });
                return [2 /*return*/];
            });
        }); });
        test('should return corresponding to dependencies chunks in the real-world example', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'components'
                    },
                    {
                        name: 'store'
                    },
                    {
                        name: 'profile',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'products',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'back-office',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'landing',
                        dependencies: {
                            components: '*'
                        }
                    },
                    {
                        name: 'host',
                        dependencies: {
                            profile: '*',
                            products: '*',
                            landing: '*',
                            'back-office': '*',
                            store: '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [
                        ['components', 'store'],
                        ['profile', 'products', 'back-office', 'landing'],
                        ['host']
                    ],
                    unprocessed: []
                });
                return [2 /*return*/];
            });
        }); });
        test('should return corresponding to dependencies chunks and unprocessed packages due to circular dependency in the real-world example', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
            var packageJsons;
            return tslib_1.__generator(this, function (_a) {
                packageJsons = [
                    {
                        name: 'components'
                    },
                    {
                        name: 'store'
                    },
                    {
                        name: 'profile',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'products',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'back-office',
                        dependencies: {
                            components: '*',
                            store: '*'
                        }
                    },
                    {
                        name: 'landing',
                        dependencies: {
                            components: '*',
                            layout: '*'
                        }
                    },
                    {
                        name: 'layout',
                        dependencies: {
                            components: '*',
                            store: '*',
                            host: '*'
                        }
                    },
                    {
                        name: 'host',
                        dependencies: {
                            profile: '*',
                            products: '*',
                            landing: '*',
                            'back-office': '*',
                            store: '*',
                            layout: '*'
                        }
                    }
                ];
                expect(dependency_utils_1.splitWorkspacesIntoDependencyGraph(packageJsons)).toEqual({
                    chunks: [
                        ['components', 'store'],
                        ['profile', 'products', 'back-office']
                    ],
                    unprocessed: [
                        ['landing', ['layout']],
                        ['layout', ['host']],
                        ['host', ['landing', 'layout']]
                    ]
                });
                return [2 /*return*/];
            });
        }); });
    });
    describe('getExternalScreen', function () {
        test('should not include anything from empty dependencies', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                peerDependencies: { grommet: '*' }
            });
            expect(externalScreen('')).toBeFalsy();
            expect(externalScreen('react')).toBeFalsy();
        });
        test('should include prod deps', function () {
            expect(dependency_utils_1.getExternalScreen({
                dependencies: { react: '*' }
            })('react')).toBeTruthy();
        });
        test('should include dev deps', function () {
            expect(dependency_utils_1.getExternalScreen({
                devDependencies: { react: '*' }
            })('react')).toBeTruthy();
        });
        test('should include peer deps', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                peerDependencies: { react: '*' }
            });
            expect(externalScreen('react')).toBeTruthy();
        });
        test('should include all deps', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                dependencies: { grommet: '*' },
                peerDependencies: { react: '*' },
                devDependencies: { rollup: '*' }
            });
            expect(externalScreen('grommet')).toBeTruthy();
            expect(externalScreen('react')).toBeTruthy();
            expect(externalScreen('rollup')).toBeTruthy();
        });
        test('should include deps with slash in the end', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                dependencies: { grommet: '*' },
                peerDependencies: { react: '*' },
                devDependencies: { rollup: '*' }
            });
            expect(externalScreen('grommet/')).toBeTruthy();
            expect(externalScreen('react/')).toBeTruthy();
            expect(externalScreen('rollup/')).toBeTruthy();
        });
        test('should include import from nested modules', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                peerDependencies: { grommet: '*' }
            });
            expect(externalScreen('grommet/utils')).toBeTruthy();
            expect(externalScreen('grommet/utils/noop')).toBeTruthy();
            expect(externalScreen('grommet/utils-react')).toBeTruthy();
        });
        test('should not include external package with the same startWith name', function () {
            var externalScreen = dependency_utils_1.getExternalScreen({
                peerDependencies: { grommet: '*' }
            });
            expect(externalScreen('grommet-utils')).toBeFalsy();
            expect(externalScreen('grommet-utils/react')).toBeFalsy();
        });
    });
});
