"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logError = void 0;
var color_utils_1 = require("./color.utils");
var common_utils_1 = require("./common.utils");
var stderr = console.error.bind(console);
function logError(err) {
    /* eslint-disable-next-line @typescript-eslint/ban-ts-comment */
    // @ts-ignore
    var error = err.error || err;
    var errorName = error.name ? error.name + " " : '';
    var errorMessage = error.message || error;
    var description = errorName ? errorName + " " + errorMessage : errorMessage;
    var message = normalizeErrorMessage(error, description);
    common_utils_1.space();
    stderr(color_utils_1.color.error(message));
    if (error.loc) {
        stderr("at " + error.loc.file + ":" + error.loc.line + ":" + error.loc.column);
    }
    common_utils_1.space();
    if (error.frame) {
        stderr(color_utils_1.color.error(error.frame));
    }
    else if (err.stack) {
        var headlessStack = error.stack.replace(message, '');
        stderr(color_utils_1.color.error(headlessStack));
    }
    common_utils_1.space();
}
exports.logError = logError;
function normalizeErrorMessage(error, description) {
    if (!error.plugin) {
        return description;
    }
    return error.plugin === 'rpt2'
        ? "(typescript) " + description
        : "(" + error.plugin + " plugin) " + description;
}
