"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findWorkspacePackageDir = exports.findWorkspaceRootDir = exports.updateYarnWorkspacesDeclaration = exports.includePackageIntoWorkspaces = exports.getWorkspacesInfo = exports.getWorkspacePackageSetupPath = exports.getWorkspacesFromDeclaration = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var shelljs_1 = require("shelljs");
var models_1 = require("../models");
var error_utils_1 = require("./error.utils");
var package_const_1 = require("../constants/package.const");
var isWorkspaceRoot = function (pkg) {
    return pkg.workspaces !== undefined && pkg.private;
};
var isWorkspacePackage = function (pkg) {
    return pkg.workspace && !pkg.private;
};
exports.getWorkspacesFromDeclaration = function (workspaces) {
    var _a;
    if (workspaces === undefined) {
        return [];
    }
    if ('packages' in workspaces) {
        return (_a = workspaces.packages) !== null && _a !== void 0 ? _a : [];
    }
    return workspaces;
};
exports.getWorkspacePackageSetupPath = function (packages) {
    if (packages.length === 0) {
        return 'packages';
    }
    var wildcard = packages.find(function (pkg) { return pkg[pkg.length - 1] === '*'; });
    if (wildcard !== undefined) {
        return wildcard.slice(0, wildcard.length - 2);
    }
    return packages[0].split('/').filter(Boolean).slice(0, -1).join('/');
};
function getWorkspacesInfo() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var rootDir, yarnWorkspacesJsonInfo, yarnWorkspaces;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, exports.findWorkspaceRootDir()];
                case 1:
                    rootDir = _a.sent();
                    yarnWorkspacesJsonInfo = shelljs_1.exec('yarn workspaces --json info', {
                        cwd: rootDir,
                        silent: true
                    }).stdout.trim();
                    yarnWorkspaces = JSON.parse(JSON.parse(yarnWorkspacesJsonInfo).data);
                    return [2 /*return*/, Object.keys(yarnWorkspaces).reduce(function (packages, name) { return tslib_1.__spread(packages, [
                            {
                                name: name,
                                location: path_1.default.resolve(rootDir, yarnWorkspaces[name].location)
                            }
                        ]); }, [])];
            }
        });
    });
}
exports.getWorkspacesInfo = getWorkspacesInfo;
exports.includePackageIntoWorkspaces = function (_a) {
    var e_1, _b;
    var packages = _a.packages, packageName = _a.packageName, setupPath = _a.setupPath;
    var dir = setupPath
        .split('/')
        .filter(function (segment) { return segment !== '.' && segment !== ''; })
        .join('/');
    var packageWorkspacePath = dir === '' ? packageName : dir + "/" + packageName;
    var workspacesSetupWildcard = dir === '' ? '*' : dir + "/*";
    try {
        for (var packages_1 = tslib_1.__values(packages), packages_1_1 = packages_1.next(); !packages_1_1.done; packages_1_1 = packages_1.next()) {
            var pkg = packages_1_1.value;
            if (pkg === workspacesSetupWildcard || pkg === packageWorkspacePath) {
                return packages;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (packages_1_1 && !packages_1_1.done && (_b = packages_1.return)) _b.call(packages_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return packages.concat(packageWorkspacePath);
};
exports.updateYarnWorkspacesDeclaration = function (workspaces, packages) {
    if (workspaces === undefined) {
        return packages;
    }
    if ('packages' in workspaces) {
        workspaces.packages = packages;
    }
    else {
        workspaces = packages;
    }
    return workspaces;
};
exports.findWorkspaceRootDir = function (intercept) {
    if (intercept === void 0) { intercept = false; }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        function find(possiblePath) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var isPathTooSmall, packageJsonPath, _a, _b;
                return tslib_1.__generator(this, function (_c) {
                    switch (_c.label) {
                        case 0:
                            isPathTooSmall = possiblePath.length < 10;
                            packageJsonPath = path_1.default.resolve(possiblePath, package_const_1.PACKAGE_JSON);
                            if (isPathTooSmall) {
                                return [2 /*return*/, null];
                            }
                            _a = fs_extra_1.default.existsSync(packageJsonPath);
                            if (!_a) return [3 /*break*/, 2];
                            _b = isWorkspaceRoot;
                            return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                        case 1:
                            _a = _b.apply(void 0, [_c.sent()]);
                            _c.label = 2;
                        case 2: return [2 /*return*/, _a ? possiblePath
                                : find(path_1.default.resolve(possiblePath, '..'))];
                    }
                });
            });
        }
        var dir, _a, workspaceError;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = find;
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_b.sent()])];
                case 2:
                    dir = _b.sent();
                    if (dir !== null) {
                        return [2 /*return*/, dir];
                    }
                    workspaceError = new models_1.NotFoundWorkspaceRootError();
                    if (intercept) {
                        throw workspaceError;
                    }
                    else {
                        error_utils_1.logError(workspaceError);
                        process.exit(1);
                    }
                    return [2 /*return*/];
            }
        });
    });
};
exports.findWorkspacePackageDir = function (intercept) {
    if (intercept === void 0) { intercept = false; }
    return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        function find(possiblePath) {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var packageJsonPath, pkg, isPathTooSmall, isRoot, isPackage;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            packageJsonPath = path_1.default.resolve(possiblePath, package_const_1.PACKAGE_JSON);
                            pkg = {};
                            if (!fs_extra_1.default.existsSync(packageJsonPath)) return [3 /*break*/, 2];
                            return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                        case 1:
                            pkg = _a.sent();
                            _a.label = 2;
                        case 2:
                            isPathTooSmall = possiblePath.length < 10;
                            isRoot = isWorkspaceRoot(pkg);
                            isPackage = isWorkspacePackage(pkg);
                            switch (true) {
                                case isPathTooSmall || isRoot:
                                    return [2 /*return*/, null];
                                case isPackage:
                                    return [2 /*return*/, possiblePath];
                                default:
                                    return [2 /*return*/, find(path_1.default.resolve(possiblePath, '..'))];
                            }
                            return [2 /*return*/];
                    }
                });
            });
        }
        var dir, _a, workspaceError;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = find;
                    return [4 /*yield*/, fs_extra_1.default.realpath(process.cwd())];
                case 1: return [4 /*yield*/, _a.apply(void 0, [_b.sent()])];
                case 2:
                    dir = _b.sent();
                    if (dir !== null) {
                        return [2 /*return*/, dir];
                    }
                    workspaceError = new models_1.NotFoundPackageWorkspaceError();
                    if (intercept) {
                        throw workspaceError;
                    }
                    else {
                        error_utils_1.logError(workspaceError);
                        process.exit(1);
                    }
                    return [2 /*return*/];
            }
        });
    });
};
