"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable sonarjs/no-duplicate-string */
var workspace_utils_1 = require("./workspace.utils");
describe('[utils.workspace]', function () {
    describe('getWorkspacePackageDirs', function () {
        test('should return empty array when there is no workspaces property', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration(undefined)).toEqual([]);
        });
        test('should return empty array when there is no packages inside workspaces config', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration({ packages: undefined })).toEqual([]);
        });
        test('should return empty array when workspaces property is empty array', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration([])).toEqual([]);
        });
        test('should return empty array when workspaces packages property is empty array', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration({ packages: [] })).toEqual([]);
        });
        test('should return packages when workspaces property has them', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration(['a', 'b'])).toEqual(['a', 'b']);
        });
        test('should return packages when workspaces packages property has them', function () {
            expect(workspace_utils_1.getWorkspacesFromDeclaration({ packages: ['a', 'b'] })).toEqual([
                'a',
                'b'
            ]);
        });
    });
    describe('getWorkspacePackageSetupPath', function () {
        test('should return default path with input of empty packages', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath([])).toBe('packages');
        });
        test('should return path segment of the first package in list', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['/a', 'b', 'c/', '/d/'])).toBe('');
        });
        test('should return path segments of the first package in list', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath([
                '/workspaces/private/a',
                'long/b',
                'whatever/c/',
                'folder/d/'
            ])).toBe('workspaces/private');
        });
        test('should return the same path regardless of slashes', function () {
            var result = 'workspaces/private';
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['/workspaces/private/a'])).toBe(result);
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['workspaces/private/a'])).toBe(result);
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['workspaces/private/a/'])).toBe(result);
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['/workspaces/private/a/'])).toBe(result);
        });
        test('should return path when there is one wildcard', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['workspaces/*'])).toBe('workspaces');
        });
        test('should return path when there is only one wildcard', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath(['a', 'workspaces/*', 'b'])).toBe('workspaces');
        });
        test('should return path of the first wildcard match', function () {
            expect(workspace_utils_1.getWorkspacePackageSetupPath([
                'a',
                'otherPackages/*',
                'c',
                'workspaces/*',
                'b'
            ])).toBe('otherPackages');
        });
    });
    describe('includePackageIntoWorkspaces', function () {
        test('should return the same input when it covers wildcard with empty "" path', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*'],
                packageName: 'components',
                setupPath: ''
            })).toEqual(['*']);
        });
        test('should return the same input when it covers wildcard with dot "." path', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*'],
                packageName: 'components',
                setupPath: '.'
            })).toEqual(['*']);
        });
        test('should add the package as new declaration', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*', 'packages/*'],
                packageName: 'components',
                setupPath: 'workspaces'
            })).toEqual(['*', 'packages/*', 'workspaces/components']);
        });
        test('should add the package as new declaration with "/" before setupPath', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*', 'packages/*'],
                packageName: 'components',
                setupPath: '/workspaces'
            })).toEqual(['*', 'packages/*', 'workspaces/components']);
        });
        test('should add the package as new declaration with "./" before setupPath', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*', 'packages/*'],
                packageName: 'components',
                setupPath: '/workspaces'
            })).toEqual(['*', 'packages/*', 'workspaces/components']);
        });
        test('should add the package as new declaration with "/" after setupPath', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*', 'packages/*'],
                packageName: 'components',
                setupPath: 'workspaces/'
            })).toEqual(['*', 'packages/*', 'workspaces/components']);
        });
        test('should add the package as new declaration with "/" before and after setupPath', function () {
            expect(workspace_utils_1.includePackageIntoWorkspaces({
                packages: ['*', 'packages/*'],
                packageName: 'components',
                setupPath: '/workspaces/'
            })).toEqual(['*', 'packages/*', 'workspaces/components']);
        });
    });
});
